map = {}
w, h = 0, 0
dis = 0.5

function reset(nw, nh)
  -- Generate new map
  for y = 1, nh do
    if map[y] == nil then
      table.insert(map, {})
    end
    for x = 1, nw do
      map[y][x] = 0
    end
  end

  -- Trim the map if its size was decreased
  for y, yv in ipairs(map) do
    while #map[y] > nw do
      table.remove(map[y])
    end
  end
  while #map > nh do
    table.remove(map)
  end

  -- Store the new map dimensions
  w = nw
  h = nh
end

-- Midpoint displacement algorithm (recursive)
function midpointDis(l, t, r, b, d)
  local cx = math.floor(l + (r - l)/2)
  local cy = math.floor(t + (b - t)/2)
  
  local lt = map[t][l]
  local rt = map[t][r]
  local lb = map[b][l]
  local rb = map[b][r]

  local a = (lt + rt + lb + rb)/4
  local dis = math.random()*(d*2) - d
  a = a + dis
  map[cy][cx] = a

  map[t][cx] = (lt + rt)/2
  map[b][cx] = (lb + rb)/2
  map[cy][l] = (lt + lb)/2
  map[cy][r] = (rt + rb)/2

  if r - l > 2 and b - t > 2 then
    local hd = d/2
    midpointDis(l, t, cx, cy, hd)
    midpointDis(l, cy, cx, b, hd)
    midpointDis(cx, t, r, cy, hd)
    midpointDis(cx, cy, r, b, hd)
  end
end

-- Sets the altitude values of all tiles in a range between 0 and 1
function normalize()
  local max = -math.huge
  local min = math.huge
  for y = 1, h do
    for x = 1, w  do
      local v = map[y][x]
      min = math.min(v, min)
      max = math.max(v, max) 
    end
  end
  local range = max - min
  assert(range>0)
  for y = 1, h do
    for x = 1, w do
      map[y][x] = (map[y][x] - min)/range
    end
  end
end

-- Generates a random map
function generate(nw, nh, d)
  -- Map dimensions must be power of 2 + 1
  assert(nw == nh)
  --local pow2 = (nw - 1)/2
  --assert(math.floor(pow2) == pow2)
  w, h = nw, nh
  dis = d
  reset(w, h)
  math.randomseed(os.time())
  midpointDis(1, 1, w, h, dis)
  normalize()

  imgData = love.image.newImageData(w, h)
  for y = 1, h do
    for x = 1, w do
      local v = 255*map[y][x]
      imgData:setPixel(x - 1, y - 1, v, v, v, 255)
    end
  end
  img = love.graphics.newImage(imgData)
end

function love.load()
  generate(512, 512, dis)
end

function love.draw()
  love.graphics.draw(img, 0, 0, 0, 1, 1, 0, 0)
end

function love.keypressed(key)
  if key == " " then
    generate(w, h, dis)
  end
end
		
